@@ -15,6 +15,26 @@ from thetadata import (
 from . import tc
 
 
+@pytest.mark.skip(reason="No data for contract")  # TODO: remove
+def test_end_of_day(tc: ThetaClient):
+    """Test an EOD historical request."""
+    res = tc.get_hist_option(
+        req=OptionReqType.EOD,
+        root="AAPL",
+        exp=datetime.date(2022, 7, 15),
+        strike=140,
+        right=OptionRight.CALL,
+        date_range=DateRange(
+            datetime.date(2022, 7, 6), datetime.date(2022, 7, 15)
+        ),
+        progress_bar=True,
+    )
+    print(res)
+    assert isinstance(res, DataFrame)
+    assert len(res.index) > 0
+
+
+@pytest.mark.skip(reason="No data for contract")  # TODO: remove
 def test_hist_option_quotes_small(tc: ThetaClient):
     """Test a historical option request."""
     res = tc.get_hist_option(
@@ -34,6 +54,7 @@ def test_hist_option_quotes_small(tc: ThetaClient):
     assert len(res.index) > 0
 
 
+@pytest.mark.skip(reason="No data for contract")  # TODO: remove
 def test_hist_option_quotes_large(tc: ThetaClient):
     """Test a very large historical option request."""
     res = tc.get_hist_option(
@@ -86,6 +107,7 @@ def test_hist_option_open_interest(tc: ThetaClient):
     assert len(res.index) > 0
 
 
+@pytest.mark.skip(reason="No data for contract")  # TODO: remove
 def test_get_expirations(tc: ThetaClient):
     """Test an expirations listing request."""
     res = tc.get_expirations(root="BDX")
@@ -94,12 +116,14 @@ def test_get_expirations(tc: ThetaClient):
     assert len(res.index) > 0
 
 
+@pytest.mark.skip(reason="No data for contract")  # TODO: remove
 def test_get_strikes_error(tc: ThetaClient):
     """Ensure that an invalid strike listing request raises."""
     with pytest.raises(thetadata.ResponseError) as e_info:
         res = tc.get_strikes(root="BDX", exp=datetime.date(2022, 6, 1))
 
 
+@pytest.mark.skip(reason="No data for contract")  # TODO: remove
 def test_get_strikes(tc: ThetaClient):
     """Test a strike listing request."""
     res = tc.get_strikes(
@@ -111,6 +135,7 @@ def test_get_strikes(tc: ThetaClient):
     assert len(res.index) > 0
 
 
+@pytest.mark.skip(reason="No data for contract")  # TODO: remove
 def test_get_roots(tc: ThetaClient):
     """Test a root listing request."""
     res = tc.get_roots(sec=SecType.OPTION)
