@@ -1,9 +1,13 @@
 """Module that contains Theta Client class."""
-from typing import Optional, Callable
+import warnings
+from threading import Thread
+from time import sleep
+from typing import Optional
 from contextlib import contextmanager
-from functools import wraps
-from datetime import datetime, date
+
 import socket
+
+from pandas import DataFrame
 from tqdm import tqdm
 import pandas as pd
 
@@ -13,8 +17,7 @@ from .parsing import (
     TickBody,
     ListBody,
 )
-
-from .exceptions import ResponseError
+from .terminal import check_download, launch_terminal
 
 _NOT_CONNECTED_MSG = "You must esetablish a connection first."
 
@@ -32,16 +35,31 @@ def _format_date(dt: date) -> str:
 class ThetaClient:
     """A high-level, blocking client used to fetch market data."""
 
-    def __init__(self, port: int = 11000, timeout: Optional[float] = 10):
+    def __init__(self, port: int = 11000, timeout: Optional[float] = 60,
+                 username: str = None, passwd: str = None, auto_update: bool = True):
         """Construct a client instance to interface with market data.
 
         :param port: The port number specified in the Theta Terminal config
         :param timeout: The max number of seconds to wait for a response before
             throwing a TimeoutError
+        :param username: Theta Data username / email. When specified with a
+            password, this class will launch the terminal in headless mode.
+        :param passwd: Theta Data password. When specified with a username,
+            this class will launch the terminal in headless mode.
+        :param auto_update: If true, this class will automatically download the
+            latest terminal version each time this class is instantiated. If
+            false, the terminal will use the current jar terminal file. If none
+            exists, it will download the latest version.
         """
         self.port: int = port
         self.timeout = timeout
         self._server: Optional[socket.socket] = None  # None while disconnected
+        if username is not None and passwd is not None:
+            check_download(auto_update)
+            Thread(target=launch_terminal, args=[username, passwd]).start()
+        else:
+            warnings.warn("You must specify a username and passwd to access data!"
+                          " If you have a terminal already running, you can ignore this message.")
 
     @contextmanager
     def connect(self):
@@ -50,6 +68,7 @@ class ThetaClient:
         :raises ConnectionRefusedError: If the connection failed.
         :raises TimeoutError: If the timeout is set and has been reached.
         """
+
         try:
             self._server = socket.socket()
             self._server.connect(("localhost", self.port))
@@ -67,19 +86,22 @@ class ThetaClient:
         assert self._server is not None, _NOT_CONNECTED_MSG
 
         # receive body data in parts
-        PART_SIZE = 4096  # 4 KiB recommended for most machines
+        MAX_PART_SIZE = 512  # 4 KiB recommended for most machines
 
         buffer = bytearray(n_bytes)
         bytes_downloaded = 0
-        for i in tqdm(
-            range(0, n_bytes, PART_SIZE),
-            desc="Downloading",
-            disable=not progress_bar,
-        ):
-            bytes_downloaded += PART_SIZE
-            part = self._server.recv(PART_SIZE)
-            buffer[i : i + PART_SIZE] = part
 
+        # tqdm disable=True is slow bc it still calls __new__, which takes nearly 4ms
+        range_ = range(0, n_bytes, MAX_PART_SIZE)
+        iterable = tqdm(range_, desc="Downloading") if progress_bar else range_
+
+        for i in iterable:
+            part_size = min(MAX_PART_SIZE, n_bytes - bytes_downloaded)
+            bytes_downloaded += part_size
+            part = self._server.recv(part_size)
+            buffer[i : i + part_size] = part
+
+        assert bytes_downloaded == n_bytes
         return buffer
 
     def kill(self) -> None:
@@ -88,7 +110,7 @@ class ThetaClient:
         All subsequent requests will timeout until the Terminal is restarted.
         """
         assert self._server is not None, _NOT_CONNECTED_MSG
-        kill_msg = f"ID=0&MSG_CODE={MessageType.KILL.value}\n"
+        kill_msg = f"MSG_CODE={MessageType.KILL.value}\n"
         self._server.sendall(kill_msg.encode("utf-8"))
 
     def get_hist_option(
@@ -99,6 +121,7 @@ class ThetaClient:
         strike: float,
         right: OptionRight,
         date_range: DateRange,
+        interval_size: int = 0,
         progress_bar: bool = False,
     ) -> pd.DataFrame:
         """
@@ -110,6 +133,7 @@ class ThetaClient:
         :param strike:         The strike price in USD, rounded to 1/10th of a cent.
         :param right:          The right of an option.
         :param date_range:     The dates to fetch.
+        :param interval_size:        The interval size in milliseconds. Applicable only to OHLC & QUOTE requests.
         :param progress_bar:   Print a progress bar displaying download progress.
 
         :return:               The requested data as a pandas DataFrame.
@@ -123,18 +147,17 @@ class ThetaClient:
         end_fmt = _format_date(date_range.end)
 
         # send request
-        request_id = 0
-        hist_msg = f"ID={request_id}&MSG_CODE={MessageType.HIST.value}&START_DATE={start_fmt}&END_DATE={end_fmt}&root={root}&exp={exp_fmt}&strike={strike}&right={right.value}&sec={SecType.OPTION.value}&req={req.value}\n"
+        hist_msg = f"MSG_CODE={MessageType.HIST.value}&START_DATE={start_fmt}&END_DATE={end_fmt}&root={root}&exp={exp_fmt}&strike={strike}&right={right.value}&sec={SecType.OPTION.value}&req={req.value}&IVL={interval_size}\n"
         self._server.sendall(hist_msg.encode("utf-8"))
 
         # parse response header
-        header: Header = Header.parse(self._server.recv(20))
+        header_data = self._server.recv(20)
+        header: Header = Header.parse(hist_msg, header_data)
 
         # parse response body
         body_data = self._recv(header.size, progress_bar=progress_bar)
-        body: TickBody = TickBody.parse(header, body_data)
-
-        return body.ticks
+        body: DataFrame = TickBody.parse(hist_msg, header, body_data)
+        return body
 
     # LISTING DATA
 
@@ -147,11 +170,10 @@ class ThetaClient:
         :raises ResponseError: If the request failed.
         """
         assert self._server is not None, _NOT_CONNECTED_MSG
-        req_id = 1
-        out = f"MSG_CODE={MessageType.ALL_EXPIRATIONS.value}&ID={req_id}&root={root}\n"
+        out = f"MSG_CODE={MessageType.ALL_EXPIRATIONS.value}&root={root}\n"
         self._server.send(out.encode("utf-8"))
-        header = Header.parse(self._server.recv(20))
-        body = ListBody.parse(header, self._recv(header.size))
+        header = Header.parse(out, self._server.recv(20))
+        body = ListBody.parse(out, header, self._recv(header.size), dates=True)
         return body.lst
 
     def get_strikes(self, root: str, exp: date) -> pd.Series:
@@ -166,11 +188,10 @@ class ThetaClient:
         assert self._server is not None, _NOT_CONNECTED_MSG
         assert isinstance(exp, date)
         exp_fmt = _format_date(exp)
-        req_id = 1
-        out = f"MSG_CODE={MessageType.ALL_STRIKES.value}&ID={req_id}&root={root}&exp={exp_fmt}\n"
+        out = f"MSG_CODE={MessageType.ALL_STRIKES.value}&root={root}&exp={exp_fmt}\n"
         self._server.send(out.encode("utf-8"))
-        header = Header.parse(self._server.recv(20))
-        body = ListBody.parse(header, self._recv(header.size))
+        header = Header.parse(out, self._server.recv(20))
+        body = ListBody.parse(out, header, self._recv(header.size))
         return body.lst
 
     def get_roots(self, sec: SecType) -> pd.Series:
@@ -182,11 +203,10 @@ class ThetaClient:
         :raises ResponseError: If the request failed.
         """
         assert self._server is not None, _NOT_CONNECTED_MSG
-        req_id = 1
-        out = f"MSG_CODE={MessageType.ALL_ROOTS.value}&ID={req_id}&sec={sec.value}\n"
+        out = f"MSG_CODE={MessageType.ALL_ROOTS.value}&sec={sec.value}\n"
         self._server.send(out.encode("utf-8"))
-        header = Header.parse(self._server.recv(20))
-        body = ListBody.parse(header, self._recv(header.size))
+        header = Header.parse(out, self._server.recv(20))
+        body = ListBody.parse(out, header, self._recv(header.size))
         return body.lst
 
     # LIVE DATA
@@ -217,12 +237,32 @@ class ThetaClient:
         exp_fmt = _format_date(exp)
 
         # send request
-        request_id = 0
-        hist_msg = f"ID={request_id}&MSG_CODE={MessageType.LAST.value}&root={root}&exp={exp_fmt}&strike={strike}&right={right.value}&sec={SecType.OPTION.value}&req={req.value}\n"
+        hist_msg = f"MSG_CODE={MessageType.LAST.value}&root={root}&exp={exp_fmt}&strike={strike}&right={right.value}&sec={SecType.OPTION.value}&req={req.value}\n"
         self._server.sendall(hist_msg.encode("utf-8"))
 
         # parse response
-        header: Header = Header.parse(self._server.recv(20))
-        body: TickBody = TickBody.parse(header, self._recv(header.size))
+        header: Header = Header.parse(hist_msg, self._server.recv(20))
+        body: DataFrame = TickBody.parse(
+            hist_msg, header, self._recv(header.size)
+        )
+
+        return body
+
+    def get_req(
+        self,
+        req: str,
+    ) -> pd.DataFrame:
+        assert self._server is not None, _NOT_CONNECTED_MSG
+        # send request
+        print(req)
+        self._server.sendall(req.encode("utf-8"))
+
+        # parse response header
+        header_data = self._server.recv(20)
+        header: Header = Header.parse(req, header_data)
+
+        # parse response body
+        body_data = self._recv(header.size, progress_bar=False)
+        body: DataFrame = TickBody.parse(req, header, body_data)
+        return body
 
-        return body.ticks
